@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's constant types and some special constants. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,10 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <string>
-#include <stdexcept>
-#include <iostream>
-
 #include "constant.h"
 #include "numeric.h"
 #include "ex.h"
@@ -31,6 +27,10 @@
 #include "utils.h"
 #include "inifcns.h"
 
+#include <iostream>
+#include <stdexcept>
+#include <string>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(constant, basic,
@@ -45,7 +45,7 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(constant, basic,
 
 // public
 
-constant::constant() : basic(&constant::tinfo_static), ef(0), serial(next_serial++), domain(domain::complex)
+constant::constant() : ef(nullptr), serial(next_serial++), domain(domain::complex)
 {
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
@@ -57,20 +57,20 @@ constant::constant() : basic(&constant::tinfo_static), ef(0), serial(next_serial
 // public
 
 constant::constant(const std::string & initname, evalffunctype efun, const std::string & texname, unsigned dm)
-  : basic(&constant::tinfo_static), name(initname), ef(efun), serial(next_serial++), domain(dm)
+  : name(initname), ef(efun), serial(next_serial++), domain(dm)
 {
 	if (texname.empty())
-		TeX_name = "\\mbox{" + name + "}";
+		TeX_name = "\\mathrm{" + name + "}";
 	else
 		TeX_name = texname;
 	setflag(status_flags::evaluated | status_flags::expanded);
 }
 
 constant::constant(const std::string & initname, const numeric & initnumber, const std::string & texname, unsigned dm)
-  : basic(&constant::tinfo_static), name(initname), ef(0), number(initnumber), serial(next_serial++), domain(dm)
+  : name(initname), ef(nullptr), number(initnumber), serial(next_serial++), domain(dm)
 {
 	if (texname.empty())
-		TeX_name = "\\mbox{" + name + "}";
+		TeX_name = "\\mathrm{" + name + "}";
 	else
 		TeX_name = texname;
 	setflag(status_flags::evaluated | status_flags::expanded);
@@ -80,25 +80,24 @@ constant::constant(const std::string & initname, const numeric & initnumber, con
 // archiving
 //////////
 
-constant::constant(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst) {}
-
-ex constant::unarchive(const archive_node &n, lst &sym_lst)
+void constant::read_archive(const archive_node &n, lst &sym_lst)
 {
 	// Find constant by name (!! this is bad: 'twould be better if there
 	// was a list of all global constants that we could search)
 	std::string s;
 	if (n.find_string("name", s)) {
 		if (s == Pi.name)
-			return Pi;
+			*this = Pi;
 		else if (s == Catalan.name)
-			return Catalan;
+			*this = Catalan;
 		else if (s == Euler.name)
-			return Euler;
+			*this = Euler;
 		else
 			throw (std::runtime_error("unknown constant '" + s + "' in archive"));
 	} else
 		throw (std::runtime_error("unnamed constant in archive"));
 }
+GINAC_BIND_UNARCHIVER(constant);
 
 void constant::archive(archive_node &n) const
 {
@@ -132,7 +131,7 @@ void constant::do_print_latex(const print_latex & c, unsigned level) const
 void constant::do_print_python_repr(const print_python_repr & c, unsigned level) const
 {
 	c.s << class_name() << "('" << name << "'";
-	if (TeX_name != "\\mbox{" + name + "}")
+	if (TeX_name != "\\mathrm{" + name + "}")
 		c.s << ",TeX_name='" << TeX_name << "'";
 	c.s << ')';
 }
@@ -149,9 +148,9 @@ bool constant::info(unsigned inf) const
 		return inherited::info(inf);
 }
 
-ex constant::evalf(int level) const
+ex constant::evalf() const
 {
-	if (ef!=0) {
+	if (ef!=nullptr) {
 		return ef();
 	} else {
 		return number.evalf();
@@ -216,7 +215,8 @@ bool constant::is_equal_same_type(const basic & other) const
 
 unsigned constant::calchash() const
 {
-	hashvalue = golden_ratio_hash((p_int)tinfo() ^ serial);
+	const void* typeid_this = (const void*)typeid(*this).name();
+	hashvalue = golden_ratio_hash((uintptr_t)typeid_this ^ serial);
 
 	setflag(status_flags::hash_calculated);
 
